package e2_PAC;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class ColorController {

	private ColorModel model;
	private RGBColorController superController;
	private ArrayList<ColorView> views;
	
	public ColorController(RGBColorController sController, Integer value) {
		model = new ColorModel(value);
		superController = sController;
		views = new ArrayList<ColorView>();
	}

	void setValue(Integer value, Object caller) {
		model.setValue(value);
		for (ColorView view : views) {
			view.update(this, caller);
		}
	}

	void setValue(String value, Object caller) {
		try {
			this.setValue(Integer.parseInt(value), caller);
		} catch (NumberFormatException e) {

		}
	}
	
	public Integer getValue() {
		return model.getValue();
	}
	
	public String toHex() {
		return model.toHex();
	}
	
	public String toHex(Integer minLength) {
		return model.toHex(minLength);
	}
	
	public RGBColorController getSuperController() {
		return superController;
	}

	public void addView(ColorView view) {
		views.add(view);
	}

	public void removeView(ColorView view) {
		views.remove(view);
	}

	/**
	 * Binds a slider to the integer.
	 */
	public void bind(final JSlider slider) {
		// view for the color
		this.addView(new ColorView() {
			JSlider that = slider;

			@Override
			public void update(ColorController controller, Object caller) {
				if (!that.equals(caller)) {
					that.setValue(controller.getValue());
				}
			}
		});
		// Listener for slider changes
		slider.addChangeListener(new ChangeListener() {
			JSlider that = slider;

			@Override
			public void stateChanged(ChangeEvent event) {
				ColorController.this.setValue(that.getValue(), that);
			}
		});
		// Update the slider to the integer value
		slider.setValue(getValue());
	}

	/**
	 * Binds a text field to the integer.
	 */
	public void bind(final JTextField textField) {
		// view for the color
		this.addView(new ColorView() {
			JTextField that = textField;

			@Override
			public void update(ColorController controller, Object caller) {
				if (!that.equals(caller)) {
					that.setText(controller.getValue().toString());
				}
			}
		});
		// Listener for text field input
		textField.addActionListener(new ActionListener() {
			JTextField that = textField;

			@Override
			public void actionPerformed(ActionEvent arg0) {
				ColorController.this.setValue(that.getText(), that);
			}
		});
		// Update the text field to the integer value
		textField.setText(getValue().toString());
	}

}
